import sys

sys.path.insert(0, "./transformers_repo/src/")
sys.path.insert(0, "./peft_repo/src/")

import torch
from dataset_library.factory import get_dataset
from transformers import (
    AutoModelForVision2Seq,
    AutoProcessor,
    TrainingArguments,
    modeling_utils,
)

from trl import (
    ModelConfig,
    TrlParser,
    get_kbit_device_map,
    get_quantization_config,
)

from utils.args import (
    ContinualScriptArguments,
    ContinualModelConfig,
    ContinualRegularizationArguments,
)
from typing import TYPE_CHECKING


if __name__ == "__main__":
    parser = TrlParser(
        (
            ContinualScriptArguments,
            TrainingArguments,
            ContinualModelConfig,
            ContinualRegularizationArguments,
        )
    )
    script_args, training_args, model_args, reg_args = parser.parse_args_and_config()
    # for type hint
    if TYPE_CHECKING:
        script_args = ContinualScriptArguments()
        training_args = TrainingArguments()
        model_args = ContinualModelConfig()
        reg_args = ContinualRegularizationArguments()

    training_args.gradient_checkpointing_kwargs = dict(use_reentrant=False)
    training_args.remove_unused_columns = False

    from model_library.factory import get_model

    model, processor, collate_fn_for_train, collate_fn_for_evaluate = get_model(
        model_args=model_args, training_args=training_args
    )
    ################
    # Dataset
    ################

    from utils.accelerator import create_accelerator_and_postprocess

    accelerator = create_accelerator_and_postprocess(training_args)

    accelerator.print(model)

    for dataset_name in script_args.dataset_name:
        dataset = get_dataset(dataset_name)
        model = accelerator.prepare_model(model, evaluation_mode=True)

        model.eval()

        from torch.utils.data import DataLoader

        bs = 3 if dataset_name not in ["scienceqa"] else 1
        accelerator.print("Batch size:", bs)

        val_dataloader = DataLoader(
            dataset[script_args.dataset_generation_split],
            batch_size=bs,
            collate_fn=collate_fn_for_evaluate,
        )
        val_dataloader = accelerator.prepare_data_loader(val_dataloader)
        from utils.evaluate_tool import evaluate_rouge, evaluate_save

        evaluate_save(model, val_dataloader, processor, accelerator)
